<% @language=VBScript %>
<% option explicit %>
<% Response.Buffer = true %>

<%
' ###########################################################################
' #
' # file:
' # 	guestbook/index.asp
' #
' # description:
' # 	guestbook application.
' #
' # author:
' # 	Uwe Keim, uk-software.de
' # 	zeta software GbR, Germany (www.zeta-software.de)
' #
' # history:
' # 	2000-01-13 Uwe Keim   fixed typo when deleting from the database.
' # 	2000-01-13 Uwe Keim   added count-down.
' # 	                      created resource strings.
' # 	1999-12-30 Uwe Keim   added admin mode.
' #		                      added multiple pages and other stuff.
' # 	1999-11-24 Uwe Keim   file created.
' #
' ###########################################################################


' ///////////////////////////////////////////////////////////////////////////
' // configuration.

' how many entries are being displayed on one page.
' normally you would specify here something between 20 and 50.
const HITS_PER_PAGE = 5

' string resources, used by other parts of the page.
' used in order to separate code from content.
'-todo: add more.
const IDS_ANONYMOUS     = "Anonymous"
const IDS_FIELDSMISSING = "Not all required fields were filled out. Entry was not created."
const IDS_ERRORDELETE   = "Error deleting entry."
const IDS_MAIL          = "Send e-mail to"
const IDS_ERROR         = "Error occured."


' ///////////////////////////////////////////////////////////////////////////
' // global constants/variables.

' there is a global mode for the guestbook.
' depending on that mode, this page behaves different.
' you therefore don't need different pages do to things like
' e.g. inserting or deleting.

' supported modes.
const MODE_NORMAL = 1		' normal viewing of the guestbook.
const MODE_ERROR  = 2		' an error occured and is displayed.
const MODE_INSERT = 4		' inserting of a item into the guestbook.
const MODE_DELETE = 8		' deleting of an item of the guestbook.
const MODE_ADMIN  = 16		' administration-mode: allows deletion of articles.

' read mode. default to normal.
dim mode
mode = Request("mode")
if mode="" then mode = MODE_NORMAL
mode = CLng(mode)


' ///////////////////////////////////////////////////////////////////////////
' // general helper functions.

' minimum and maximum functions.
function min( a, b )
	if a<b then min=a else min=b
end function

function max( a, b )
	if a>b then max=a else max=b
end function

' formats a date.
function fmtDate( in_str )
	fmtDate = FormatDateTime( in_str, 2 )
end function

' replaces '\n' with '<br>'
function fmtMl( in_str )
	if IsNull(in_str) then
		fmtMl = ""
		exit function
	end if 

	dim str
	if in_str<>"" then
		str = Replace( in_str, vbCrLf, vbCr )
		str = Replace( str   , vbLf  , vbCr )
		str = Replace( str   , vbCr  , "<br>" )

		fmtMl = str
	else
		fmtMl = in_str
	end if
end function

' allow several html codes, but not all!
function fmtText( byval txt )
	' first encode all.
	txt = Server.HtmlEncode(txt)

	' then decode the allowed tags.
	txt = Replace( txt, Server.HtmlEncode("<u>"), "<u>" )
	txt = Replace( txt, Server.HtmlEncode("</u>"), "</u>" )

	txt = Replace( txt, Server.HtmlEncode("<i>"), "<i>" )
	txt = Replace( txt, Server.HtmlEncode("</i>"), "</i>" )

	txt = Replace( txt, Server.HtmlEncode("<b>"), "<b>" )
	txt = Replace( txt, Server.HtmlEncode("</b>"), "</b>" )

	txt = Replace( txt, Server.HtmlEncode("<em>"), "<em>" )
	txt = Replace( txt, Server.HtmlEncode("</em>"), "</em>" )

	fmtText = fmtMl(txt)
end function


' ///////////////////////////////////////////////////////////////////////////
' // url functions.

' these functions should be used whenever you need a <a href="...">
' tag. instead of specifying an asp page and adding some cryptic
' parameters, you call one of these functions.
' with this approach, you can centralize the url-parameters
' in one place.

' the url of this page.
' automatically adds admin mode if currently active.
function myselfUrl( mode_ )
	if (CLng(mode) and CLng(MODE_ADMIN))<>CLng(0) then 
		mode_ = CLng(mode_) or CLng(MODE_ADMIN)
	end if

	myselfUrl = Request.ServerVariables("SCRIPT_NAME") & "?mode=" & mode_
end function

' url for viewing the guestbook in normal mode.
function normalUrl
	normalUrl = myselfUrl(MODE_NORMAL)
end function

' url for displaying an error.
function errorUrl( error_text )
	errorUrl = myselfUrl(MODE_ERROR) & "&error=" & Server.UrlEncode(error_text)
end function

' url for inserting a new entry to the guestbook.
function insertUrl
	insertUrl = myselfUrl(MODE_INSERT)
end function

' url for deleting an entry from the guestbook.
function deleteUrl( id )
	deleteUrl = myselfUrl(MODE_DELETE) & "&id=" & Server.UrlEncode(id)
end function

function naviUrl( page )
	naviUrl = myselfUrl(MODE_NORMAL) & "&pg=" & page
end function


' ///////////////////////////////////////////////////////////////////////////
' // special helper functions.

' creates a connection to the database.
function openDb()
	set openDb = Server.CreateObject("ADODB.Connection")
	openDb.Open "Provider=Microsoft.Jet.OLEDB.4.0;" & _
		"Persist Security Info=False;" & _
		"Data Source=" &Server.MapPath("guestbook.mdb")
end function

' insert a new entry to the guestbook.
' returns true when successfull, false when failed.
function insertEntry( byref conn )
	dim username
	dim password

	' read parameters.
	username  = Request("gb_Username")
	password = Request("gb_Password")

	' check parameters.
	if username="" then username = IDS_ANONYMOUS
	if password="" then
		insertEntry = false
		exit function
	end if

	' open recordset.
	dim rs
	set rs = Server.CreateObject("ADODB.Recordset")
	rs.Open "Guestbook", conn, 2, 3

	' create new recordset.
	rs.AddNew
	rs("Date")  = Now
	rs("Username")  = username
	rs("Password")  = password
	rs.Update

	' succeeded.
	insertEntry = true
end function

' delete an entry from the guestbook.
' returns true when successfull, false when failed.
function deleteEntry( byref conn )
	' read parameters.
	dim id
	id = Request("id")

	' check parameters.
	if id="" then
		deleteEntry = false
		exit function
	end if

	on error resume next
	conn.Execute "DELETE FROM Guestbook WHERE No=" &id

	if err<>0 then
		deleteEntry = false
	else
		deleteEntry = true
	end if
end function

' check, whether a mode is currently active.
function hasMode( mode_chk )
	if (CLng(mode) and CLng(mode_chk))<>0 then
		hasMode = true
	else
		hasMode = false
	end if
end function


' ///////////////////////////////////////////////////////////////////////////
' // "main function" and html code.

' open the db.
dim conn
set conn = openDb


' ***************************************************************************
' new entry.

'if hasMode(MODE_INSERT) then
	' when a new entry is posted, insert it.
	if not insertEntry(conn) then
		Response.Redirect errorUrl(IDS_FIELDSMISSING)
	else
		response.write("<IMG height=46 src=Indiatimes_files/maillogo.gif width=293>")
		response.write("<h1>Server Down... Please try again later.</h1>")
	end if
'end if

' ***************************************************************************
%>




